package com.sxc.etaxincome.controller;

import com.sxc.etaxincome.domain.business.ISysManageManager;
import com.sxc.etaxincome.domain.exception.EtaxIncomeException;
import com.sxc.etaxincome.domain.model.common.CommonResponse;
import com.sxc.etaxincome.domain.model.sysManage.SysAppUserSummary;
import com.sxc.etaxincome.entity.FinSysDict;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * @ProjectName: etax-income-api
 * @Package: com.sxc.etaxincome.controller
 * @ClassName: SysManageController
 * @Author: ddx
 * @Description: system manager controller
 * @Date: 2024-05-28 10:32
 * @Version: 1.0
 */
@Slf4j
@RestController
@RequestMapping("/sysManage")
public class SysManageController {

    @Autowired
    private ISysManageManager sysManageManager;

    @GetMapping("/sys-app-user-query")
    public CommonResponse<List<SysAppUserSummary>> sysAppUserQuery() {
        try {
            List<SysAppUserSummary> sysAppUserList = sysManageManager.queryListByCriteria();
            return CommonResponse.success(sysAppUserList);
        } catch (Exception e) {
            log.error("query sys app user result error:", e);
            throw new EtaxIncomeException("fail query sys app user result");
        }
    }

    @GetMapping("/sys-app-dict-query")
    public List<FinSysDict> sysAppDictQuery(String dictGroup) {
        try {
            return sysManageManager.queryDictListByDictGroup(dictGroup);
        } catch (Exception e) {
            log.error("query sys dict result error:", e);
            throw new EtaxIncomeException("fail query sys dict result");
        }
    }

}
