package com.sxc.etaxincome.domain.business.impl;

import com.google.common.collect.Lists;
import com.sxc.etaxincome.domain.business.IApiConfigManager;
import com.sxc.etaxincome.domain.format.ReportParamFormat;
import com.sxc.etaxincome.domain.model.report.ApiConfig;
import com.sxc.etaxincome.domain.model.report.ApiParamMapping;
import com.sxc.etaxincome.entity.EtaxReportApiConfig;
import com.sxc.etaxincome.entity.EtaxReportParamConfig;
import com.sxc.etaxincome.enums.ApiParamReadonly;
import com.sxc.etaxincome.service.IEtaxReportApiConfigService;
import com.sxc.etaxincome.service.IEtaxReportParamConfigService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

@Slf4j
@Service
public class ApiConfigManager implements IApiConfigManager {

    @Autowired
    private IEtaxReportApiConfigService etaxReportApiConfigService;

    @Autowired
    private IEtaxReportParamConfigService etaxReportParamConfigService;

    @Override
    public ApiConfig getApiConfig(String apiCode) {
        EtaxReportParamConfig etaxReportParamConfigCriteria = new EtaxReportParamConfig();
        etaxReportParamConfigCriteria.setApiCode(apiCode);
        etaxReportParamConfigCriteria.setReadonly(ApiParamReadonly.NO.getCode());
        List<EtaxReportParamConfig> etaxReportParamConfigList = etaxReportParamConfigService.queryList(etaxReportParamConfigCriteria);
        List<ApiParamMapping> apiParamMappingList = Lists.newArrayListWithExpectedSize(etaxReportParamConfigList.size());
        for (EtaxReportParamConfig etaxReportParamConfig : etaxReportParamConfigList) {
            apiParamMappingList.add(ApiParamMapping.builder().apiParamName(this.renderApiParamName(etaxReportParamConfig)).etaxParamName(etaxReportParamConfig.getEtaxParamName()).build());
        }
        EtaxReportApiConfig etaxReportApiConfigCriteria = new EtaxReportApiConfig();
        etaxReportApiConfigCriteria.setApiCode(apiCode);
        EtaxReportApiConfig etaxReportApiConfig = etaxReportApiConfigService.queryOne(etaxReportApiConfigCriteria);
        return ApiConfig.builder()
                .apiCode(etaxReportApiConfig.getApiCode())
                .apiUrl(etaxReportApiConfig.getApiUrl())
                .apiParamMappingList(apiParamMappingList).build();
    }

    private String renderApiParamName(EtaxReportParamConfig etaxReportParamConfig) {
        String tableCode = etaxReportParamConfig.getTableCode();
        String row = etaxReportParamConfig.getLocationRow();
        String col = etaxReportParamConfig.getLocationCol();
        return ReportParamFormat.formatReportParam(tableCode, row, col);
    }
}
