package com.sxc.etaxincome.domain.business.impl;

import com.alibaba.fastjson.JSON;
import com.alicp.jetcache.anno.CacheType;
import com.alicp.jetcache.anno.Cached;
import com.sxc.etaxincome.domain.Version;
import com.sxc.etaxincome.domain.business.IVersionManager;
import com.sxc.etaxincome.domain.model.common.AggregateVersion;
import com.sxc.etaxincome.domain.model.common.ApiVersion;
import com.sxc.etaxincome.domain.model.common.BuildVersion;
import com.sxc.framework.common.file.FileUtil;
import com.sxc.framework.common.json.JSONUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

/**
 * @author caosongqing
 * @date 2022/12/12
 * @description
 */
@Slf4j
@Service
public class VersionManager implements IVersionManager {

    private static final int VERSION_LENGTH = 10;

    @Override
    @Cached(name = "income-invoice:version:version", expire = 36000, cacheType = CacheType.LOCAL)
    public String version() {
        String startWith = "j";
        int subRevisionLength = VERSION_LENGTH;
        String latestVersion;
        try {
            String versionJson = this.allVersion();
            ApiVersion apiVersion = JSON.parseObject(versionJson, ApiVersion.class);
            latestVersion = apiVersion.getLatestVersion().getVersion();
        } catch (Exception e) {
            latestVersion = "unknown";
            log.error("get version error:", e);
        }
        String revision = Version.SCM_REVISION;
        int lengthRevision = revision.length();
        String subRevision = StringUtils.substring(revision, lengthRevision - subRevisionLength, lengthRevision);
        String version = String.format("%s_%s_%s", startWith, latestVersion, subRevision);
        log.info("version:{}", version);
        return version;
    }

    @Override
    @Cached(name = "income-invoice:version:buildVersion", expire = 36000, cacheType = CacheType.LOCAL)
    public String buildVersion() {
        BuildVersion buildVersion = BuildVersion.builder()
                .projectVersion(Version.PROJECT_VERSION)
                .revision(Version.SCM_REVISION)
                .branch(Version.SCM_BRANCH)
                .date(Version.TIMESTAMP).build();
        return JSONUtil.toString(buildVersion);
    }

    @Override
    @Cached(name = "income-invoice:version:allVersion", expire = 36000, cacheType = CacheType.LOCAL)
    public String allVersion() {
        return FileUtil.readContent("version/version.json");
    }

    @Override
    @Cached(name = "income-invoice:version:latestVersion", expire = 36000, cacheType = CacheType.LOCAL)
    public String latestVersion() {
        String version = this.allVersion();
        ApiVersion apiVersion = JSON.parseObject(version, ApiVersion.class);
        BuildVersion buildVersion = BuildVersion.builder()
                .projectVersion(Version.PROJECT_VERSION)
                .revision(Version.SCM_REVISION)
                .branch(Version.SCM_BRANCH)
                .date(Version.TIMESTAMP)
                .build();
        AggregateVersion aggregateVersion = AggregateVersion.builder()
                .apiVersionLog(apiVersion.getLatestVersion())
                .buildVersion(buildVersion)
                .build();
        return JSONUtil.toString(aggregateVersion);
    }
}
