package com.sxc.etaxincome.domain.command.analysis;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.sxc.etaxincome.criteria.FinRealLogCriteria;
import com.sxc.etaxincome.domain.constants.CommonConstants;
import com.sxc.etaxincome.domain.model.common.DateTimeRange;
import com.sxc.etaxincome.domain.model.report.AnalysisCriteria;
import com.sxc.etaxincome.domain.support.DateProvider;
import com.sxc.etaxincome.entity.FinRealLog;
import com.sxc.etaxincome.service.IFinRealLogService;
import com.sxc.framework.common.json.JSONUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Comparator;
import java.util.List;

/**
 * @ProjectName: etax-income-api
 * @Package: com.sxc.etaxincome.domain.command.analysis
 * @ClassName: QueryIncomeSuccessAnalysisCommand
 * @Author: ddx
 * @Description:
 * @Date: 2024-06-21 17:21
 * @Version: 1.0
 */

@Component(value = "queryIncomeSuccessAnalysisCommand")
public class QueryIncomeSuccessAnalysisCommand implements IAnalysisCommand {

    @Autowired
    private IFinRealLogService finRealLogService;

    @Override
    public String handle(AnalysisCriteria analysisCriteria) {
        DateTimeRange dateTimeRange = DateProvider.today();
        FinRealLogCriteria finRealLogCriteria = FinRealLogCriteria.builder()
                .userCode(analysisCriteria.getUsername())
                .zoneCode(analysisCriteria.getAreaCode())
                .resCode(CommonConstants.SUCCESS)
                .dateTimeStart(dateTimeRange.getStartDateTime())
                .dateTimeEnd(dateTimeRange.getEndDateTime())
                .build();
        int start = 1;
        int limit = 100;
        IPage<FinRealLog> realLogPage = finRealLogService.queryPageListOrderByRequestTimeDesc(finRealLogCriteria, start, limit);
        List<FinRealLog> realLogList = realLogPage.getRecords();
        if (CollectionUtils.isNotEmpty(realLogList)) {
            realLogList.sort(Comparator.comparing(FinRealLog::getReqDate));
            return JSONUtil.toString(realLogList);
        }
        return "[]";
    }

}
