package com.sxc.etaxincome.domain.command.query;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.nacos.api.config.annotation.NacosValue;
import com.google.common.base.Stopwatch;
import com.sxc.etaxincome.common.util.RetryHttpHelper;
import com.sxc.etaxincome.domain.business.IApiConfigManager;
import com.sxc.etaxincome.domain.business.IResultTransferManager;
import com.sxc.etaxincome.domain.constants.EtaxReportConstants;
import com.sxc.etaxincome.domain.exception.EtaxIncomeException;
import com.sxc.etaxincome.domain.model.common.CommonRequestWrapper;
import com.sxc.etaxincome.domain.model.common.CommonResponse;
import com.sxc.etaxincome.domain.model.cookie.TokenNecessary;
import com.sxc.etaxincome.domain.model.report.*;
import com.sxc.etaxincome.domain.template.ResponseHandlerTemplate;
import com.sxc.etaxincome.entity.EtaxReportType;
import com.sxc.etaxincome.enums.ApiCode;
import com.sxc.etaxincome.service.IEtaxReportTypeService;
import com.sxc.framework.common.httpclient.ResponseContent;
import com.sxc.framework.common.json.JSONUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Map;
import java.util.concurrent.TimeUnit;

import static com.sxc.etaxincome.domain.constants.EtaxReportConstants.*;

@Slf4j
@Component(value = "queryReportItemQueryCommand")
public class QueryReportItemQueryCommand implements IQueryCommand {

    @NacosValue(value = "${url.etax.report}", autoRefreshed = true)
    private String etaxReportUrl;

    @Autowired
    private IApiConfigManager apiConfigManager;

    @Autowired
    private IResultTransferManager resultTransfarManager;

    @Autowired
    private IEtaxReportTypeService etaxReportTypeService;

    @Override
    public CommonResponse<IReportResponse> execute(CommonRequestWrapper commonRequestWrapper) {
        Stopwatch stopwatch = Stopwatch.createStarted();
        ApiConfig apiConfig = apiConfigManager.getApiConfig(ApiCode.QUERY_REPORT_ITEM.getCode());
        JSONObject requestDataJsonObject = new JSONObject();
        String requestDataStr = commonRequestWrapper.getData();
        String requestSerialNo = commonRequestWrapper.getRequestSerialNo();
        log.info("queryReportItem requestSerialNo:{} data:{}", requestSerialNo, requestDataStr);
        QueryReportItemRequestWrapper queryReportItemRequestWrapper = JSON.parseObject(requestDataStr, QueryReportItemRequestWrapper.class);
        TokenNecessary tokenNecessary = queryReportItemRequestWrapper.decodeToken();
        String token = tokenNecessary.getToken();
        requestDataJsonObject.put(API_PARAM_TOKEN, token);
        requestDataJsonObject.put(API_PARAM_AREA_CODE, queryReportItemRequestWrapper.getArea());
        String test = queryReportItemRequestWrapper.getTest();
        if (StringUtils.isNotBlank(test)) {
            requestDataJsonObject.put(API_PARAM_TEST, test);
        }
        if (StringUtils.isNotBlank(queryReportItemRequestWrapper.getDeclareDateStart())) {
            requestDataJsonObject.put("skssqq", queryReportItemRequestWrapper.getDeclareDateStart());
        }
        if (StringUtils.isNotBlank(queryReportItemRequestWrapper.getDeclareDateEnd())) {
            requestDataJsonObject.put("skssqz", queryReportItemRequestWrapper.getDeclareDateEnd());
        }
        if (StringUtils.isNotBlank(queryReportItemRequestWrapper.getSbuuid())) {
            requestDataJsonObject.put("sbuuid", queryReportItemRequestWrapper.getSbuuid());
        }
        if (StringUtils.isNotBlank(queryReportItemRequestWrapper.getReportType())) {
            JSONArray reportIdJsonArray = new JSONArray();
            reportIdJsonArray.add(queryReportItemRequestWrapper.getReportType());
            requestDataJsonObject.put("reportIdList", reportIdJsonArray);
        }
        try {
            String etaxReportUrlFull = etaxReportUrl + apiConfig.getApiUrl();
            log.info("url:{}", etaxReportUrlFull);
            String requestJson = JSONUtil.toString(requestDataJsonObject);
            log.info("request:{}", requestJson);
            ResponseContent responseContent = RetryHttpHelper.retryPostJsonEntity(etaxReportUrlFull, requestJson);
            String response = responseContent.getContent();
            log.info("response:{}", response);
            stopwatch.stop();
            log.info("query report item elapsed time:{}ms", stopwatch.elapsed(TimeUnit.MILLISECONDS));
            return new ResponseHandlerTemplate<IReportResponse>() {
                @Override
                public CommonResponse<IReportResponse> handleSuccess(JSONObject jsonObject) {
                    EtaxReportType etaxReportTypeCriteria = new EtaxReportType();
                    etaxReportTypeCriteria.setReportId(queryReportItemRequestWrapper.getReportType());
                    EtaxReportType etaxReportType = etaxReportTypeService.queryOne(etaxReportTypeCriteria);
                    if (etaxReportType == null) {
                        return CommonResponse.error("未查询到报表信息");
                    }
                    String tableCode = etaxReportType.getTableCode();
                    String formData = jsonObject.getString(EtaxReportConstants.ETAX_RESPONSE_DATA_KEY);
                    ReportItemResponse reportItemResponse = new ReportItemResponse();
                    Map<String, String> reportItemMap = resultTransfarManager.transferToMap(formData, tableCode);
                    reportItemResponse.putAll(reportItemMap);
                    return CommonResponse.success(reportItemResponse);
                }
            }.handle(response);
        } catch (Exception e) {
            log.error("query report item error:", e);
            throw new EtaxIncomeException(e);
        }
    }
}
